<?php
namespace Ep\Woo\View\Model;

use Ep\Woo\View\AddToCart\AbstractAddToCart;
use WC_Product;
use WC_Product_Grouped;
use WC_Product_Variable;

/**
 * Class AddToCart
 *
 * @package Ep\View\Model
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class AddToCart
{
    /** @var WC_Product */
    private $product;
    /** @var string */
    private $price;
    /** @var bool */
    private $isChild;
    /** @var string */
    private $formId;

    /** @var array */
    private $availableVariations;
    /** @var AddToCart[] */
    private $children;
    /** @var bool */
    private $purchasable;
    /** @var string */
    private $quantityInput;
    /** @var string */
    private $stock;
    /** @var bool */
    private $hasVariations;

    /**
     * AddToCart constructor.
     *
     * @param WC_Product $product
     * @param bool $isChild
     */
    public function __construct(WC_Product $product, bool $isChild = false)
    {
        $this->product = $product;
        $this->isChild = $isChild;
    }

    /**
     * @return string
     */
    public function getAjax_action(): string
    {
        return AbstractAddToCart::AJAX_ACTION;
    }

    /**
     * @return array
     */
    public function getAvailable_variations(): array
    {
        if (null === $this->availableVariations) {
            /** @var WC_Product_Variable $product */
            $product = $this->product;
            $this->availableVariations = $this->hasVariations()
                ? $product->get_available_variations()
                : [];

//            ob_end_clean();
//            var_dump($this->availableVariations); exit;
        }

        return $this->availableVariations;
    }

    /**
     * @return string
     */
    public function getButton_text(): string
    {
        return $this->product->single_add_to_cart_text();
    }

    /**
     * @return array|AddToCart[]
     */
    public function getChildren(): array
    {
        if (null === $this->children) {
            $this->children = [];

            if ($this->product instanceof WC_Product_Grouped) {
                foreach ($this->product->get_children() as $childId) {
                    $child = wc_get_product($childId);
                    $this->children[] = new static($child, true);
                }
            }
        }

        return $this->children;
    }

    /**
     * @return string
     */
    public function getForm_id(): string
    {
        if (null === $this->formId) {
            $this->formId = uniqid('add-to-cart_');
        }

        return $this->formId;
    }

    /**
     * @return int
     */
    public function getId(): int
    {
        return $this->product->get_id();
    }

    /**
     * @return bool
     */
    public function isIn_stock(): bool
    {
        return $this->product->is_in_stock();
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->product->get_name();
    }

    /**
     * @return string
     */
    public function getPrice(): string
    {
        if (null === $this->price) {
            $old = $GLOBALS['product'];
            $GLOBALS['product'] = $this->product;

            ob_start();
            woocommerce_template_loop_price();
            $this->price = ob_get_clean();

            $GLOBALS['product'] = $old;
        }

        return $this->price;
    }

    /**
     * @return bool
     */
    public function isPurchasable(): bool
    {
        if (null === $this->purchasable) {
            if ($this->product instanceof WC_Product_Grouped) {
                $this->purchasable = false;
                /** @var WC_Product[] $children */
                $children = array_map('wc_get_product', $this->product->get_children(''));

                foreach ($children as $child) {
                    if ($child->is_purchasable()) {
                        $this->purchasable = true;
                        break;
                    }
                }
            } else {
                $this->purchasable = $this->product->is_purchasable();
            }
        }

        return $this->purchasable;
    }

    /**
     * @return string
     */
    public function getQuantity_input(): string
    {
        if (null === $this->quantityInput) {
            ob_start();

            if ($this->isChild) {
                $args['input_name'] = sprintf('quantity[%d]', $this->product->get_id());
                $args['input_value'] = 0;
                $args['min_value'] = 0;
            } else {
                $args['input_value'] = isset($_POST['quantity']) ? wc_stock_amount($_POST['quantity']) : $this->product->get_min_purchase_quantity();
                $args['min_value'] = apply_filters('woocommerce_quantity_input_min', $this->product->get_min_purchase_quantity(), $this->product);
            }

            if ($this->product instanceof WC_Product_Variable) {
                $args['max_value'] = '';
            } else {
                $args['max_value'] = apply_filters('woocommerce_quantity_input_max', $this->product->get_max_purchase_quantity(), $this->product);
            }

            $args['orig_max_value'] = $args['max_value'];

            woocommerce_quantity_input($args);
            $this->quantityInput = ob_get_clean();
        }

        return $this->quantityInput;
    }

    /**
     * @return string
     */
    public function getReset_text(): string
    {
        return __('Clear', 'woocommerce');
    }

    /**
     * @return string
     */
    public function getStock(): string
    {
        if (null === $this->stock) {
            $this->stock = wc_get_stock_html($this->product);
        }

        return $this->stock;
    }

    /**
     * @return string
     */
    public function getSku(): string
    {
        return $this->product->get_sku();
    }

    /**
     * @return string
     */
    public function getType(): string
    {
        return $this->product->get_type();
    }

    /**
     * @return string
     */
    public function getUnavailable_message(): string
    {
        return __('This product is currently out of stock and unavailable.', 'woocommerce');
    }

    /**
     * @return string
     */
    public function getUrl(): string
    {
        return get_permalink($this->product->get_id());
    }

    /**
     * @return bool
     */
    public function hasVariations(): bool
    {
        if (null === $this->hasVariations) {
            $variationThreshold = apply_filters('woocommerce_ajax_variation_threshold', 30, $this->product);
            $this->hasVariations = count($this->product->get_children()) <= $variationThreshold;
        }

        return $this->hasVariations;
    }

    /**
     * @return bool
     */
    public function getVisible(): bool
    {
        return $this->product->is_visible();
    }
}