<?php
namespace Ep\Woo\View\Model;

use WC_Product;

/**
 * View model za cijenu.
 *
 * @package Mishe\View\Model
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Price
{
    /** @var WC_Product */
    private $product;
    /** @var string */
    private $suffix;

    /**
     * Price constructor.
     *
     * @param WC_Product $product
     */
    public function __construct(WC_Product $product)
    {
        $this->product = $product;
    }

    /**
     * Da li proizvod ima cijenu.
     *
     * @return bool
     */
    public function hasPrice(): bool
    {
        return '' !== $this->product->get_price();
    }

    /**
     * Da li je proizvod na akciji.
     *
     * @return bool
     */
    public function hasSale(): bool
    {
        return $this->product->is_on_sale();
    }

    /**
     * Vraća html za cijenu kada proizvod nema cijene.
     *
     * @return string
     */
    public function getEmpty(): string
    {
        return apply_filters('woocommerce_empty_price_html', '', $this->product);
    }

    /**
     * Vraća formatiranu redovnu cijenu.
     *
     * @return string
     */
    public function getRegular(): string
    {
        return $this->format($this->getRegularPrice());
    }

    /**
     * Vraća oznaku redovne cijene.
     *
     * @return string
     */
    public function getRegularLabel(): string
    {
        return __('Price:', 'woocommerce');
    }

    /**
     * Vraća cijenu za prikaz.
     *
     * @return float
     */
    protected function getRegularPrice(): float
    {
        $price = $this->hasSale()
            ? $this->product->get_regular_price()
            : $this->product->get_price();

        return wc_get_price_to_display($this->product, [
            'price' => $price,
        ]);
    }

    /**
     * Vraća formatiranu akcijsku cijenu.
     *
     * @return string
     */
    public function getSale(): string
    {
        return $this->format($this->getSalePrice());
    }

    /**
     * Vraća oznaku akcijske cijene.
     *
     * @return string
     */
    public function getSaleLabel(): string
    {
        return __('Sale Price:', 'mishe');
    }

    /**
     * Vraća akcijsku cijenu.
     *
     * @return float
     */
    protected function getSalePrice(): float
    {
        return wc_get_price_to_display($this->product);
    }

    /**
     * Vraća dodatak nakon cijene, npr. 'Bez PDV-a'.
     *
     * @return string
     */
    public function getSuffix(): string
    {
        if (null === $this->suffix) {
            $this->suffix = strip_tags($this->product->get_price_suffix());
        }

        return $this->suffix;
    }

    /**
     * Vraća formatiranu cijenu
     *
     * @param float|null $price
     *
     * @return string
     */
    private function format(float $price): string
    {
        $currency = get_woocommerce_currency_symbol();
        $decimal_separator = wc_get_price_decimal_separator();
        $thousand_separator = wc_get_price_thousand_separator();
        $decimals = wc_get_price_decimals();
        $format = get_woocommerce_price_format();

        $negative = $price < 0;
        $price = apply_filters('raw_woocommerce_price', floatval($negative ? $price * -1 : $price));

        $formatted = number_format($price, $decimals, $decimal_separator, $thousand_separator);
        $formatted = apply_filters('formatted_woocommerce_price', $formatted, $price, $decimals, $decimal_separator, $thousand_separator);

        $formatted = sprintf(
            $format,
            sprintf('<span class="price__currency">%s</span>', $currency),
            $formatted
        );

        return $formatted;
    }
}